module model_type_mod
use sep_mod
implicit none
type vel_model
   integer :: n1,n2,n3
   real o1,o2,o3,d1,d2,d3
   real,allocatable,dimension(:,:,:) :: vals
   integer,allocatable,dimension(:,:,:) :: layer
end type
contains

subroutine create_new_model(in,out,nbeg,nend)
  type(vel_model) :: in,out
  integer :: nbeg,nend
  allocate(out%vals(in%n1+nbeg+nend,in%n2,in%n3),out%layer(in%n1+nbeg+nend,in%n2,in%n3))
  out%n1=in%n1+nbeg+nend
  out%n2=in%n2
  out%n3=in%n3
  out%o1=in%o1
  out%o2=in%o2
  out%o3=in%o3
  out%d1=in%d1
  out%d2=in%d2
  out%d3=in%d3
  out%vals=-1
  out%layer=-1
  out%vals(nbeg+1:nbeg+in%n1,:,:)=in%vals(:,:,:)
  out%layer(nbeg+1:nbeg+in%n1,:,:)=in%layer(:,:,:)
end subroutine

subroutine clean_model(in)
  type(vel_model) :: in
  deallocate(in%vals)
  deallocate(in%layer)
end subroutine
subroutine write_model(tag,out)
  type(vel_model) :: out
  character(len=*) :: tag
  integer :: ierr,i3
  
  ierr=sep_put_data_axis_par(trim(tag),1,out%n1,out%o1,out%d1,"z")
  ierr=sep_put_data_axis_par(trim(tag),2,out%n2,out%o2,out%d2,"x")
  ierr=sep_put_data_axis_par(trim(tag),3,out%n3,out%o3,out%d3,"y")
  do i3=1,out%n3
    ierr=srite(trim(tag),out%vals(:,:,i3),out%n2*out%n1*4)
  end do
end subroutine
subroutine allocate_model(out,n1,o1,d1,n2,o2,d2,n3,o3,d3,val)
  type(vel_model) :: out
  integer :: n1,n3,n2
  real :: o1,o2,o3,d1,d2,d3,val
  allocate(out%vals(n1,n2,n3),out%layer(n1,n2,n3))
  out%n1=n1
  out%n2=n2
  out%n3=n3
  out%o1=o1
  out%o2=o2
  out%o3=o3
  out%d1=d1
  out%d2=d2
  out%d3=d3
  out%vals=val
  out%layer=0
end subroutine


subroutine cat_it(t2,t1,v)
  character(len=*) :: t1,t2
  integer :: v
  if(v<10) then
    write(t2,"(a,i1)") trim(t1),v
  else if(v<100) then
    write(t2,"(a,i2)") trim(t1),v
  else if(v<1000) then
    write(t2,"(a,i3)") trim(t1),v
  else 
    call seperr("only can handle up to 999")
  end if
end subroutine
subroutine cat_it_2(t2,t1,v,v2)
  character(len=*) :: t1,t2
  integer :: v,v2
  if(v<10) then
    write(t2,"(a,i1)") trim(t1),v
  else if(v<100) then
    write(t2,"(a,i2)") trim(t1),v
  else if(v<1000) then
    write(t2,"(a,i3)") trim(t1),v
  else 
    call seperr("only can handle up to 999")
  end if
  if(v2<10) then
    write(t2,"(a,a,i1)") trim(t2),"_",v2
  else if(v2<100) then
    write(t2,"(a,a,i2)") trim(t2),"_",v2
  else if(v2<1000) then
    write(t2,"(a,a,i3)") trim(t2),"-",v2
  else 
    call seperr("only can handle up to 999")
  end if




end subroutine


end module